"""
`embedops_cli`
=======================================================================
* Author(s): Bailey Steinfadt, Zhi Xuen Lai
"""
from .version import (
    __version__,
)  # Import __version__ attribute so that it is visible to the setup.cfg
